function B = l2l1(A, lambda, delta, Niter)
%
%    Copyright (C) 2004-2014  Yves Goussard
%
%    This program is free software: you can redistribute it and/or modify
%    it under the terms of the GNU General Public License as published by
%    the Free Software Foundation, either version 3 of the License, or
%    (at your option) any later version.
%
%    This program is distributed in the hope that it will be useful,
%    but WITHOUT ANY WARRANTY; without even the implied warranty of
%    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%    GNU General Public License for more details.
%
%    You should have received a copy of the GNU General Public License
%    along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
% Dbruitage d'une image par pnalisation L2L1 des diffrences premires
%
% Syntaxe : B = l2l1(A, lambda, delta, Niter)
%
%           A : image  traiter (tableau 2D rel)
%           lambda : paramtre de rgularisation
%           delta : facteur d'chelle de la fonction de pnalisation
%           Niter : iombre maximum d'itrations
%
%           B : image dbruite (mme taille que A)
%

[Nbl, Nbc] = size(A);
if exist('Niter') ~= 1
  Niter = 100;
end


% Initialisations diverses

beta = 1.7;
alpha =  1.99 * delta;
W = [0.01, 1, 1];

% Diagonalisation de la matrice des diffrences premires, direction i

Di = zeros(Nbl, Nbc);
if Nbl > 1
  col1 = zeros(Nbl, 1);
  col1(1) = 1;
  col1(Nbl) = -1;
  Di = fft(col1) * ones(1, Nbc);
end

% Diagonalisation de la matrice des diffrences premires, direction j

Dj = zeros(Nbl, Nbc);
if Nbc > 1
  lig1 = zeros(1, Nbc);
  lig1(1) = 1;
  lig1(Nbc) = -1;
  Dj = ones(Nbl, 1) * fft(lig1);
end

B = zeros(size(A));
Af = fft(fft(A, [], 1), [], 2);
MatNorm = ones(size(A)) + lambda/(2*alpha) * ...
    (W(1) +  W(2)*conj(Di).*Di + W(3)*conj(Dj).*Dj);
diff = 1;
iter = 1;

fprintf('l2l1 : estimation en cours. Itration no    ')

while (diff > 1e-05) & iter <= Niter 
% for iter = 1:Niter
  Bf = fft(fft(B, [], 1), [], 2);
  b0 = B - alpha * (B ./ sqrt(delta^2 + B.^2));
  B0 = fft(fft(b0, [], 1), [], 2);
  
  ui = B  - circshift(B, [-1, 0]); 
  bi = ui - alpha * (ui ./ sqrt(delta^2 + ui.^2)); 
  Bi = fft(fft(bi, [], 1), [], 2);

  uj = B  - circshift(B, [0, -1]);    
  bj = uj - alpha * (uj ./ sqrt(delta^2 + uj.^2)); 
  Bj = fft(fft(bj, [], 1), [], 2);

  vec = Af + lambda/(2*alpha) * ...
      (W(1) * B0 + W(2)*conj(Di).*Bi + W(3)*conj(Dj).*Bj);
  Bf = (1-beta) * Bf + beta * vec ./ MatNorm;
  Bnew = real(ifft(ifft(Bf, [], 1), [], 2));
  diff = 100 * ((Bnew(:) - B(:))' * (Bnew(:) - B(:))) / (Bnew(:)' * Bnew(:));
  B = Bnew;
  fprintf('\b\b\b%3.0f', iter)
  iter = iter + 1;
end

fprintf('\nl2l1 : termin!\n')

return
