function [B, h] = lapgauss(A, sigma, percentile)
%
%    Copyright (C) 2004-2014  Yves Goussard
%
%    This program is free software: you can redistribute it and/or modify
%    it under the terms of the GNU General Public License as published by
%    the Free Software Foundation, either version 3 of the License, or
%    (at your option) any later version.
%
%    This program is distributed in the hope that it will be useful,
%    but WITHOUT ANY WARRANTY; without even the implied warranty of
%    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%    GNU General Public License for more details.
%
%    You should have received a copy of the GNU General Public License
%    along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
% Dtection de contour par filtrage linaire par un masque de type 
%  laplacien d'une gaussienne  suivi par un seuillage pour ne conserver
% que les valeurs situes dans le percentile suprieur de la sortie.
%
% Syntaxe : [B, h] = lapgauss(A, sigma, percentile)
%
%           A : image  traiter (tableau 2D rel)
%           sigma : cart-type de la gaussienne, en pixels ou fraction de
%                   pixels (rel)
%           percentile : percentile de la sortie conserv pour afficher les
%                        contours. Dfaut : 5
%
%           B : image des contours dtects (mme taille que A)
%           h : PSF du filter dtecteur
%

if exist('percentile') ~= 1
  percentile = 5;
end

supp = round(3*sigma);
x = -supp:supp;
[X, Y] = meshgrid(x);
R2 = X.^2 + Y.^2;

h = - (-2/sigma^2 + R2/sigma^4) .* exp(-R2/(2*sigma^2));
% h = h / sum(h(:))

B = filter2(h, A, 'same');

index = sort(B(:), 'descend');
thr = index(round(percentile * length(index) / 100));

B = seuil(B, thr);

return

